#!/usr/bin/tclsh
#
# Script: QIPRSSarse.tcl
#
#	 Copyright q.i. productions 2001
#
# This script is an add-on to the original script, which parses the
# simple Sane Magazine horoscopes file format, as defined at 
# http://www.sanemagazine.com/publish/formats/horos-0.5.dtd
#
# Please see the README.rtf file for more information on this script
# and how to use it and the RSS features added here.
#
# A full version of the licence notice is available online at
#     http://supertart.com/software/licence.html
# Write us and we'll send you stickers.
#
# Word. _m 17.03.01

## Note that we *still* avoid using namespaces, to get backwards 
## compatibility with older tcl runtimes.

 proc rssParse {data {flags NULL}} {

 ## See below for the different arrays and elements you'll be left with
 ## at the end of all of this.
 global qiRImg qiR qiRItem numItems


 ## First off, check if this is a valid RSS file. Initially, we shouldn't
 ## have to worry about this.
 ## If this is -1, it's not an rss file, or not against our criteria,
 ## which is horribly (wrongfully) crude, at the moment.
 ## That said, we probably want to do *more* validation soon.
 ## Otherwise, we'll now have access to which index of $datalist
 ## is the beginning of the content.
 
 ## There should also be a check for HTTP/1.1 404 Not Found instances here...

 set validFile [string first "rss version" $data]
 if {$validFile < 0} {
   puts "This doesn't seem to be a valid rss file."
   puts "\n\nWe're getting out of here...\n\n"
   exit
 }

 ## Now we isolate the channel from all the other junk in the file (comments,
 ## xml declarations, rss declaration, &c.)

 set qiRChannelContent [string range $data [string first "<channel>" $data] [expr [string last "</channel>" $data] + 9]]
 set endOfChannel [string last "</channel>" $data]

 #puts $qiRChannelContent

 ## We'll now get the PCDATA from our channel, it's image, items,
 ## skipHours, and skipDays.
 ## A note: qiRChannelPCDATA is the full list of elements, qiRPCDATA is each
 ## element in the channel which actually *is* pcdata

 set qiRChannelPCDATA [list "title" "description" "link" "language" "item" "rating" "image" "copyright" "pubDate" "lastBuildDate" "docs" "managingEditor" "webMaster" "skipHours" "skipDays"]
 set qiRPCDATA [list "title" "description" "link" "language" "rating" "copyright" "pubDate" "lastBuildDate" "docs" "managingEditor" "webMaster"]
 set qiRImagePCDATA [list "title" "url" "link" "width" "height" "description"]
 set qiRItemPCDATA [list "title" "link" "description"]
 set qiRSkipHoursPCDATA [list "hour"]
 set qiRSkipDaysPCDATA [list "day"]

 ## This bit sifts through the different content variables and begins
 ## pulling together a list about this particular data source.
 ## If an element hasn't been specified in the file we're reading
 ## it will return an empty variable.
 
 foreach element $qiRChannelPCDATA {
    if {[string compare $element "item"] != 0} {
        set qiR($element) [string range $qiRChannelContent [string first "<$element>" $qiRChannelContent] [string first "</$element>" $qiRChannelContent]]
        set qiR($element) [string trimright $qiR($element) <]
        regsub "<$element>" $qiR($element) "" qiR($element)
        #puts qiR($element)
        #puts $qiR($element) 
    } else {
                ## this bit just initialises $i and $n
                ## $n is also, handily, the number of items,
                ## which is set below to numItems, a global
                ## variable.
                ## startRange and endRange just grab the first item
                ## if endRange is -1, the loop is over straight away
        set n 0
        set startRange [string first "<$element>" $qiRChannelContent]
        set endRange [string first "</$element>" $qiRChannelContent]
                ## set a (r)evisedChannelContent variable for this branch
        set rChannelContent $qiRChannelContent
        set i $endRange

        while {$i != -1} {
          incr n
          set qiR(item$n) [string range $rChannelContent $startRange $endRange]
          set startRange [expr $endRange + 1]
          set rChannelContent [string range $rChannelContent $startRange $endOfChannel]

          set startRange [string first "<$element>" $rChannelContent]
          set endRange [string first "</$element>" $rChannelContent]
          set i $endRange
        }
    }
 }

 set numItems $n

 ## Handle the single elements now...
 ## Cleaning occurs just below here.

 set qiRImageContent $qiR(image)
 set qiRSkipHoursContent $qiR(skipHours)
 set qiRSkipDaysContent $qiR(skipDays)
          
if {[string length $qiRImageContent]>0} {
 foreach element $qiRImagePCDATA {
        set qiRImg($element) [string range $qiRImageContent [string first "<$element>" $qiRImageContent] [string first "</$element>" $qiRImageContent]]
        set qiRImg($element) [string trimright $qiRImg($element) <]
        regsub "<$element>" $qiRImg($element) "" qiRImg($element)
        #puts qiRImg($element)
        #puts $qiRImg($element)
 }
}
 
                ############################
                # This was where I had support for multiple
                # lined elements. I don't think I need
                # this any longer.
                #
                # if {[llength $qiRItem($element$n)] > 1} {}
                #  set qiRItem($element$n) [list [join $qiRItem($element$n) "\n"]]
                #
                # What DOES need to be added is the string trimright $var <
                # across all these variables to get rid of the trailing carat.
                ############################
while {$n != 0} {
  foreach element $qiRItemPCDATA {
        set qiRItem($element$n) [string range $qiR(item$n) [string first "<$element>" $qiR(item$n)] [string first "</$element>" $qiR(item$n)]]
        set qiRItem($element$n) [string trimright $qiRItem($element$n) <]
        regsub "<$element>" $qiRItem($element$n) "" qiRItem($element$n)
        #puts qiRItem($element$n)
        #puts $qiRItem($element$n)
  }
 set n [expr $n - 1]
}

        #################################################
        ## Do not touch this code below here, it's
        ## at the moment. You should only be working on
        ## the Item processing routines above. _m
        #################################################

if {[string length $qiRSkipHoursContent]>0} {
 foreach element $qiRSkipHoursContent {
        set qiRSH($element) [string range $qiRSkipHoursContent [string first "<$element>" $qiRSkipHoursContent] [string first "</$element>" $qiRSkipHoursContent]]
 }
}

if {[string length $qiRSkipDaysContent]>0} {
 foreach element $qiRSkipDaysContent {
        set qiRSD($element) [string range $qiRSkipDaysContent [string first "<$element>" $qiRSkipDaysContent] [string first "</$element>" $qiRSkipDaysContent]]
 }      
}       
        
  
## What we should be left with, at the end of all of this,
## is a large amount of variables:
## $qiR(title) : $qiR(description) : $qiR(link) : $qiR(language) : $qiR(rating) :
## $qiR(copyright) : $qiR(pubDate) : $qiR(lastBuildDate) :
## $qiR(docs) : $qiR(managingEditor) : $qiR(webMaster) : $qiRImg(title) :
## $qiRImg(url) : $qiRImg(link) : $qiRImg(width) : $qiRImg(height) :
## $qiRImg(description) : $qiRImg(link) : $qiRImg(width) : $qiRImg(height) :
## $qiRItem(title$n) : $qiRItem(link$n) : $qiRItem(description$n) :
## We'll ignore SkipHours, SkipDays and textinput for the time being.
 
}
## EOP(rssParse)

 proc toHTML {} {
	## this proc is probably sitting in the component,
	## You should not use the routine below, as it's more
	## an example than anything. You should endeavour to
	## use the template file whenever possible.

global qiRImg qiR qiRItem numItems QIPGlobalTemplateDirectory QIPGlobalHTMLRSSTemplate QIPGlobalDefaultNumItems
# need to get file, and it's channel with you open it
# (probably in w mode (not w+)
# and then puts ?channelID? "content" to it.
	set templ [open ${QIPGlobalTemplateDirectory}${QIPGlobalHTMLRSSTemplate} r]
	set template [read $templ]
	close $templ
	set QIPCounterVariable 1
	incr QIPGlobalDefaultNumItems
	set output [eval $template]
	return $output
 }
